using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Security;
using System.Collections.Specialized;
using System.Configuration.Provider;
using System.Text.RegularExpressions;
using Guushuuse.SalaryPrj.Security.Service;
using System.Collections;
using System.Globalization;
using Guushuuse.SalaryPrj.Security.DomainModel;
using System.Security.Cryptography;

namespace Guushuuse.SalaryPrj.Security.Providers
{
    /// <summary>
    ///  ASP.NET ӦóĽɫԱʸϢݿеĴ洢й
    /// </summary>
    public class HibernateMembershipProvider : MembershipProvider
    {
        private string _applicationName;
        private bool _enablePasswordReset;
        private bool _enablePasswordRetrieval;
        private int _maxInvalidPasswordAttempts;
        private int _minRequiredNonAlphanumericCharacters;
        private int _minRequiredPasswordLength;
        private int _passwordAttemptWindow;
        private MembershipPasswordFormat _passwordFormat;
        private string _passwordStrengthRegularExpression;
        private bool _requiresQuestionAndAnswer;
        private bool _requiresUniqueEmail;

        #region 

        /// <summary>
        /// ȡҪ洢ͼԱʸϢӦó
        /// </summary>
        public override string ApplicationName
        {
            get
            {
                return _applicationName;
            }
            set
            {
                if (String.IsNullOrEmpty(value))
                {
                    throw new ArgumentNullException("Provider application name not null.");
                }
                if (value.Length > 255)
                {
                    throw new ProviderException("Provider application name too long.");
                }
                _applicationName = value;

            }
        }

        /// <summary>
        /// ȡһֵָʾԱʸṩǷΪû
        /// </summary>
        public override bool EnablePasswordReset
        {
            get { return _enablePasswordReset; }
        }

        /// <summary>
        /// ȡһֵָʾԱʸṩǷΪû
        /// </summary>
        public override bool EnablePasswordRetrieval
        {
            get { return _enablePasswordRetrieval; }
        }

        /// <summary>
        /// ȡԱʸûǰЧЧʾ𰸳Դ
        /// </summary>
        public override int MaxInvalidPasswordAttempts
        {
            get { return _maxInvalidPasswordAttempts; }
        }

        /// <summary>
        /// ȡЧбַ
        /// </summary>
        public override int MinRequiredNonAlphanumericCharacters
        {
            get { return _minRequiredNonAlphanumericCharacters; }
        }

        /// <summary>
        /// ȡҪС
        /// </summary>
        public override int MinRequiredPasswordLength
        {
            get { return _minRequiredPasswordLength; }
        }

        /// <summary>
        /// ȡԱʸû֮ǰЧЧʾ𰸳Դķ
        /// </summary>
        public override int PasswordAttemptWindow
        {
            get { return _passwordAttemptWindow; }
        }

        /// <summary>
        /// ȡһֵʾڳԱʸݿд洢ĸʽ
        /// </summary>
        public override MembershipPasswordFormat PasswordFormat
        {
            get { return _passwordFormat; }
        }

        /// <summary>
        /// ȡڼʽ
        /// </summary>
        public override string PasswordStrengthRegularExpression
        {
            get { return _passwordStrengthRegularExpression; }
        }

        /// <summary>
        /// ȡһֵָʾԱʸṩǷΪҪûڽúͼʱشʾ
        /// </summary>
        public override bool RequiresQuestionAndAnswer
        {
            get { return _requiresQuestionAndAnswer; }
        }

        /// <summary>
        /// ȡһֵָʾԱʸṩǷΪҪÿûΨһĵʼַ
        /// </summary>
        public override bool RequiresUniqueEmail
        {
            get { return _requiresUniqueEmail; }
        }

        #endregion 

        #region 

        /// <summary>
        /// ʹ ASP.NET ӦóļֵָʼԱʸṩ
        /// </summary>
        /// <param name="name"></param>
        /// <param name="config"></param>
        public override void Initialize(string name, NameValueCollection config)
        {
            if (config == null)
            {
                throw new ArgumentNullException("config");
            }

            if (String.IsNullOrEmpty(name))
            {
                name = "HibernateMembershipProvider";
            }

            if (String.IsNullOrEmpty(config["description"]))
            {
                config.Remove("description");
                config.Add("description", "GFC.Security Membership Provider");
            }

            base.Initialize(name, config);

            this._enablePasswordRetrieval = SecUtility.GetBooleanValue(config, "enablePasswordRetrieval", false);
            this._enablePasswordReset = SecUtility.GetBooleanValue(config, "enablePasswordReset", true);
            this._requiresQuestionAndAnswer = SecUtility.GetBooleanValue(config, "requiresQuestionAndAnswer", true);
            this._requiresUniqueEmail = SecUtility.GetBooleanValue(config, "requiresUniqueEmail", true);
            this._maxInvalidPasswordAttempts = SecUtility.GetIntValue(config, "maxInvalidPasswordAttempts", 5, false, 0);
            this._passwordAttemptWindow = SecUtility.GetIntValue(config, "passwordAttemptWindow", 10, false, 0);
            this._minRequiredPasswordLength = SecUtility.GetIntValue(config, "minRequiredPasswordLength", 7, false, 128);
            this._minRequiredNonAlphanumericCharacters = SecUtility.GetIntValue(config, "minRequiredNonalphanumericCharacters", 1, true, 128);

            this._passwordStrengthRegularExpression = config["passwordStrengthRegularExpression"];
            if (this._passwordStrengthRegularExpression != null)
            {
                this._passwordStrengthRegularExpression = this._passwordStrengthRegularExpression.Trim();
                if (this._passwordStrengthRegularExpression.Length != 0)
                {
                    try
                    {
                        new Regex(this._passwordStrengthRegularExpression);
                    }
                    catch (ArgumentException e)
                    {
                        throw new ProviderException(e.Message, e);
                    }
                }
            }
            else
            {
                this._passwordStrengthRegularExpression = String.Empty;
            }

            this._applicationName = config["applicationName"];
            if (String.IsNullOrEmpty(this._applicationName))
            {
                this._applicationName = SecUtility.GetDefaultAppName();
            }

            if (this._applicationName.Length > 255)
            {
                throw new ProviderException("Provider application name is too long, max length is 255.");
            }

            if (!ServiceLocator.ApplicationService.ApplicationExists(this._applicationName))
            {
                ServiceLocator.ApplicationService.CreateApplication(this._applicationName);
            }

            string strTemp = config["passwordFormat"];
            if (strTemp == null)
            {
                strTemp = "Hashed";
            }

            switch (strTemp)
            {
                case "Clear":
                    this._passwordFormat = MembershipPasswordFormat.Clear;
                    break;
                case "Encrypted":
                    this._passwordFormat = MembershipPasswordFormat.Encrypted;
                    break;
                case "Hashed":
                    this._passwordFormat = MembershipPasswordFormat.Hashed;
                    break;
                default:
                    throw new ProviderException("Bad password format.");
            }

            if (this._passwordFormat == MembershipPasswordFormat.Hashed && this._enablePasswordRetrieval)
            {
                throw new ProviderException("Provider cannot retrieve hashed password.");
            }

            config.Remove("applicationName");
            config.Remove("enablePasswordRetrieval");
            config.Remove("enablePasswordReset");
            config.Remove("requiresQuestionAndAnswer");
            config.Remove("requiresUniqueEmail");
            config.Remove("maxInvalidPasswordAttempts");
            config.Remove("passwordAttemptWindow");
            config.Remove("passwordFormat");
            config.Remove("name");
            config.Remove("description");
            config.Remove("minRequiredPasswordLength");
            config.Remove("minRequiredNonalphanumericCharacters");
            config.Remove("passwordStrengthRegularExpression");

            if (config.Count > 0)
            {
                string attribUnrecognized = config.GetKey(0);
                if (!String.IsNullOrEmpty(attribUnrecognized))
                {
                    throw new ProviderException("Provider unrecognized attribute: " + attribUnrecognized);
                }
            }
        }

        /// <summary>
        /// ޸û
        /// </summary>
        /// <param name="username"></param>
        /// <param name="oldPassword"></param>
        /// <param name="newPassword"></param>
        /// <returns></returns>
        public override bool ChangePassword(string username, string oldPassword, string newPassword)
        {
            SecUtility.CheckParameter(ref username, true, true, true, 255, "username");
            SecUtility.CheckParameter(ref oldPassword, true, true, false, 128, "oldPassword");
            SecUtility.CheckParameter(ref newPassword, true, true, false, 128, "newPassword");

            string salt;
            MembershipPasswordFormat passwordFormat;

            if (!CheckPassword(username, oldPassword, false, false, out salt, out passwordFormat))
            {
                return false;
            }

            if (newPassword.Length < this._minRequiredPasswordLength)
            {
                throw new ArgumentException("New Password too short.");
            }

            int nonAlphanumericCharacters = 0;

            for (int i = 0; i < newPassword.Length; i++)
            {
                if (!char.IsLetterOrDigit(newPassword, i))
                {
                    nonAlphanumericCharacters++;
                }
            }

            if (nonAlphanumericCharacters < this._minRequiredNonAlphanumericCharacters)
            {
                throw new ArgumentException("Password need more non alpha numeric chars.");
            }

            if ((this._passwordStrengthRegularExpression.Length > 0) && !Regex.IsMatch(newPassword, this._passwordStrengthRegularExpression))
            {
                throw new ArgumentException("Password does not match regular expression.");
            }

            string pass = EncodePassword(newPassword, passwordFormat, salt);

            if (pass.Length > 128)
            {
                throw new ArgumentException("Membership password too long.");
            }

            ValidatePasswordEventArgs args = new ValidatePasswordEventArgs(username, newPassword, false);
            this.OnValidatingPassword(args);
            if (args.Cancel)
            {
                if (args.FailureInformation != null)
                {
                    throw args.FailureInformation;
                }
                throw new ArgumentException("Membership custom password validation failure.");
            }

            int status = ServiceLocator.UserService.UpdatePassword(this._applicationName, username, pass, salt, passwordFormat,
                DateTime.UtcNow);

            if (status != 0)
            {
                string errText = GetExceptionText(status);

                if (IsStatusDueToBadPassword(status))
                {
                    throw new MembershipPasswordException(errText);
                }
                throw new ProviderException(errText);
            }

            return true;
        }

        /// <summary>
        /// ³Աʸݿûʾʹ
        /// </summary>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <param name="newPasswordQuestion"></param>
        /// <param name="newPasswordAnswer"></param>
        /// <returns></returns>
        public override bool ChangePasswordQuestionAndAnswer(string username, string password,
            string newPasswordQuestion, string newPasswordAnswer)
        {
            SecUtility.CheckParameter(ref username, true, true, true, 255, "username");
            SecUtility.CheckParameter(ref password, true, true, false, 128, "password");

            string salt;
            MembershipPasswordFormat passwordFormat;

            if (!CheckPassword(username, password, false, false, out salt, out passwordFormat))
            {
                return false;
            }

            SecUtility.CheckParameter(ref newPasswordQuestion, this.RequiresQuestionAndAnswer, this.RequiresQuestionAndAnswer, false, 255, "newPasswordQuestion");

            if (newPasswordAnswer != null)
            {
                newPasswordAnswer = newPasswordAnswer.Trim();
            }

            SecUtility.CheckParameter(ref newPasswordAnswer, this.RequiresQuestionAndAnswer, this.RequiresQuestionAndAnswer, false, 128, "newPasswordAnswer");

            string answer;
            if (!String.IsNullOrEmpty(newPasswordAnswer))
            {
                answer = EncodePassword(newPasswordAnswer.ToLower(CultureInfo.InvariantCulture), passwordFormat, salt);
            }
            else
            {
                answer = newPasswordAnswer;
            }

            SecUtility.CheckParameter(ref answer, this._requiresQuestionAndAnswer, this._requiresQuestionAndAnswer, false, 128, "newPasswordAnswer");

            int status = ServiceLocator.UserService.ChangePasswordQuestionAndAnswer(this._applicationName, username, newPasswordQuestion, answer);

            if (status != 0)
            {
                throw new ProviderException(GetExceptionText(status));
            }

            return true;
        }

        /// <summary>
        /// Աʸݿһû
        /// </summary>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <param name="email"></param>
        /// <param name="passwordQuestion"></param>
        /// <param name="passwordAnswer"></param>
        /// <param name="isApproved"></param>
        /// <param name="providerUserKey"></param>
        /// <param name="status"></param>
        /// <returns></returns>
        public override MembershipUser CreateUser(string username, string password, string email,
            string passwordQuestion, string passwordAnswer, bool isApproved, object providerUserKey,
            out MembershipCreateStatus status)
        {
            if (!SecUtility.ValidateParameter(ref password, true, true, false, 128))
            {
                status = MembershipCreateStatus.InvalidPassword;
                return null;
            }

            string salt = GenerateSalt();
            string pass = EncodePassword(password, this._passwordFormat, salt);

            if (pass.Length > 128)
            {
                status = MembershipCreateStatus.InvalidPassword;
                return null;
            }

            if (passwordAnswer != null)
            {
                passwordAnswer = passwordAnswer.Trim();
            }

            string answer;
            if (!String.IsNullOrEmpty(passwordAnswer))
            {
                if (passwordAnswer.Length > 128)
                {
                    status = MembershipCreateStatus.InvalidAnswer;
                    return null;
                }
                answer = EncodePassword(passwordAnswer.ToLower(CultureInfo.InvariantCulture), this._passwordFormat, salt);
            }
            else
            {
                answer = passwordAnswer;
            }

            if (!SecUtility.ValidateParameter(ref answer, this._requiresQuestionAndAnswer, true, false, 128))
            {
                status = MembershipCreateStatus.InvalidAnswer;
                return null;
            }

            if (!SecUtility.ValidateParameter(ref username, true, true, true, 255))
            {
                status = MembershipCreateStatus.InvalidUserName;
                return null;
            }

            if (!SecUtility.ValidateParameter(ref email, this._requiresUniqueEmail, this._requiresUniqueEmail, false, 255))
            {
                status = MembershipCreateStatus.InvalidEmail;
                return null;
            }

            if (!SecUtility.ValidateParameter(ref passwordQuestion, this._requiresQuestionAndAnswer, true, false, 255))
            {
                status = MembershipCreateStatus.InvalidQuestion;
                return null;
            }

            if ((providerUserKey != null) && !(providerUserKey is Int32))
            {
                status = MembershipCreateStatus.InvalidProviderUserKey;
                return null;
            }


            if (password.Length < this._minRequiredPasswordLength)
            {
                status = MembershipCreateStatus.InvalidPassword;
                return null;
            }

            int nonAlphanumericCharacters = 0;

            for (int i = 0; i < password.Length; i++)
            {
                if (!char.IsLetterOrDigit(password, i))
                {
                    nonAlphanumericCharacters++;
                }
            }

            if (nonAlphanumericCharacters < this._minRequiredNonAlphanumericCharacters)
            {
                status = MembershipCreateStatus.InvalidPassword;
                return null;
            }

            if ((this._passwordStrengthRegularExpression.Length > 0) && !Regex.IsMatch(password, this._passwordStrengthRegularExpression))
            {
                status = MembershipCreateStatus.InvalidPassword;
                return null;
            }

            ValidatePasswordEventArgs args = new ValidatePasswordEventArgs(username, password, true);
            this.OnValidatingPassword(args);
            if (args.Cancel)
            {
                status = MembershipCreateStatus.InvalidPassword;
                return null;
            }

            DateTime currentTimeUtc = this.RoundToSeconds(DateTime.UtcNow);

            int userID;

            status = (MembershipCreateStatus)CreateUser(this._applicationName, username, pass, salt, email,
                passwordQuestion, answer, isApproved, this._requiresUniqueEmail, this._passwordFormat, currentTimeUtc, out userID);

            if (status != 0)
            {
                return null;
            }

            currentTimeUtc = currentTimeUtc.ToLocalTime();
            return new MembershipUser(this.Name, username, userID, email, passwordQuestion, null, isApproved, false, currentTimeUtc, currentTimeUtc, currentTimeUtc, currentTimeUtc, new DateTime(1754, 1, 1));

        }

        /// <summary>
        /// ӳԱʸݿɾûĳԱʸϢ
        /// </summary>
        /// <param name="username"></param>
        /// <param name="deleteAllRelatedData"></param>
        /// <returns></returns>
        public override bool DeleteUser(string username, bool deleteAllRelatedData)
        {
            SecUtility.CheckParameter(ref username, true, true, true, 255, "username");

            int status = ServiceLocator.UserService.DeleteUser(this._applicationName, username, deleteAllRelatedData);

            if (status != 0)
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// سԱʸûļϣûĵʼֶַΰָĵʼַ
        /// </summary>
        /// <param name="emailToMatch"></param>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="totalRecords"></param>
        /// <returns></returns>
        public override MembershipUserCollection FindUsersByEmail(string emailToMatch, int pageIndex,
            int pageSize, out int totalRecords)
        {
            SecUtility.CheckParameter(ref emailToMatch, false, false, false, 128, "emailToMatch");

            MembershipUserCollection membershipUsers = new MembershipUserCollection();

            if (pageIndex < 0)
            {
                throw new ArgumentException("PageIndex bad.");
            }

            if (pageSize < 1)
            {
                throw new ArgumentException("PageSize bad.");
            }

            long lBound = (long)pageIndex * pageSize;
            long uBound = lBound + pageSize - 1;

            if (uBound > Int32.MaxValue)
            {
                throw new ArgumentException("PageIndex,pageSize bad.");
            }

            IList users = ServiceLocator.UserService.FindUsersByEmail(this._applicationName, emailToMatch, pageIndex,
                pageSize, out totalRecords);

            foreach (User user in users)
            {
                MembershipUser membershipUser = new MembershipUser(this.Name, user.Username, user.ID, user.Email,
                    user.PasswordQuestion, user.Comment, user.IsApproved, user.IsLockedOut, user.CreateDate.ToLocalTime(),
                    user.LastLoginDate.ToLocalTime(), user.LastActivityDate.ToLocalTime(),
                    user.LastPasswordChangedDate.ToLocalTime(), user.LastLockoutDate.ToLocalTime());
                membershipUsers.Add(membershipUser);
            }

            return membershipUsers;
        }

        /// <summary>
        /// ȡһԱʸûļϣЩûûҪƥָû
        /// </summary>
        /// <param name="usernameToMatch"></param>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="totalRecords"></param>
        /// <returns></returns>
        public override MembershipUserCollection FindUsersByName(string usernameToMatch,
            int pageIndex, int pageSize, out int totalRecords)
        {
            SecUtility.CheckParameter(ref usernameToMatch, true, true, false, 255, "usernameToMatch");

            MembershipUserCollection membershipUsers = new MembershipUserCollection();

            if (pageIndex < 0)
            {
                throw new ArgumentException("PageIndex bad.");
            }

            if (pageSize < 1)
            {
                throw new ArgumentException("PageSize bad.");
            }

            long lBound = (long)pageIndex * pageSize;
            long uBound = lBound + pageSize - 1;

            if (uBound > Int32.MaxValue)
            {
                throw new ArgumentException("PageIndex,pageSize bad.");
            }

            IList users = ServiceLocator.UserService.FindUsersByName(this._applicationName, usernameToMatch, pageIndex,
                pageSize, out totalRecords);

            foreach (User user in users)
            {
                MembershipUser membershipUser = new MembershipUser(this.Name, user.Username, user.ID, user.Email,
                    user.PasswordQuestion, user.Comment, user.IsApproved, user.IsLockedOut, user.CreateDate.ToLocalTime(),
                    user.LastLoginDate.ToLocalTime(), user.LastActivityDate.ToLocalTime(),
                    user.LastPasswordChangedDate.ToLocalTime(), user.LastLockoutDate.ToLocalTime());
                membershipUsers.Add(membershipUser);
            }

            return membershipUsers;
        }

        /// <summary>
        /// ȡԱʸݿûļ
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="totalRecords"></param>
        /// <returns></returns>
        public override MembershipUserCollection GetAllUsers(int pageIndex, int pageSize,
            out int totalRecords)
        {
            MembershipUserCollection membershipUsers = new MembershipUserCollection();

            if (pageIndex < 0)
            {
                throw new ArgumentException("PageIndex bad.");
            }

            if (pageSize < 1)
            {
                throw new ArgumentException("PageSize bad.");
            }

            long lBound = (long)pageIndex * pageSize;
            long uBound = lBound + pageSize - 1;

            if (uBound > Int32.MaxValue)
            {
                throw new ArgumentException("PageIndex,pageSize bad.");
            }

            long lTotalRecords;

            IList users = ServiceLocator.UserService.GetAllUsers(this._applicationName, pageIndex,
                pageSize, out lTotalRecords);

            totalRecords = Convert.ToInt32(lTotalRecords);

            foreach (User user in users)
            {
                MembershipUser membershipUser = new MembershipUser(this.Name, user.Username, user.ID, user.Email,
                    user.PasswordQuestion, user.Comment, user.IsApproved, user.IsLockedOut, user.CreateDate.ToLocalTime(),
                    user.LastLoginDate.ToLocalTime(), user.LastActivityDate.ToLocalTime(),
                    user.LastPasswordChangedDate.ToLocalTime(), user.LastLockoutDate.ToLocalTime());
                membershipUsers.Add(membershipUser);
            }

            return membershipUsers;
        }

        /// <summary>
        /// صǰʸӦóû
        /// </summary>
        /// <returns></returns>
        public override int GetNumberOfUsersOnline()
        {
            return ServiceLocator.UserService.GetNumberOfUsersOnline(this._applicationName,
                Membership.UserIsOnlineTimeWindow, DateTime.UtcNow);
        }

        /// <summary>
        /// ӳԱʸݿָⷵû
        /// </summary>
        /// <param name="username"></param>
        /// <param name="answer"></param>
        /// <returns></returns>
        public override string GetPassword(string username, string passwordAnswer)
        {
            if (!this._enablePasswordRetrieval)
            {
                throw new NotSupportedException("Membership password retrieval not supported.");
            }

            SecUtility.CheckParameter(ref username, true, true, true, 255, "username");

            string answer = GetEncodedPasswordAnswer(username, passwordAnswer);

            SecUtility.CheckParameter(ref answer, this._requiresQuestionAndAnswer, this._requiresQuestionAndAnswer, false, 128, "passwordAnswer");

            MembershipPasswordFormat passwordFormat = MembershipPasswordFormat.Clear;
            int status = 0;
            string pass = ServiceLocator.UserService.GetPassword(this._applicationName, username, answer, this._requiresQuestionAndAnswer,
                this._maxInvalidPasswordAttempts, this._passwordAttemptWindow, out passwordFormat, out status, DateTime.UtcNow);

            if (pass == null)
            {
                string errText = this.GetExceptionText(status);

                if (this.IsStatusDueToBadPassword(status))
                {
                    throw new MembershipPasswordException(errText);
                }

                throw new ProviderException(errText);
            }

            return UnEncodePassword(pass, passwordFormat);

        }

        /// <summary>
        /// ԴȡԱʸûϢ
        /// </summary>
        /// <param name="username"></param>
        /// <param name="userIsOnline"></param>
        /// <returns></returns>
        public override MembershipUser GetUser(string username, bool userIsOnline)
        {
            SecUtility.CheckParameter(ref username, true, false, true, 255, "username");

            User user = ServiceLocator.UserService.GetUserByName(this._applicationName, username, userIsOnline, DateTime.UtcNow);

            if (user != null)
            {
                return new MembershipUser(this.Name, user.Username, user.ID, user.Email,
                    user.PasswordQuestion, user.Comment, user.IsApproved, user.IsLockedOut, user.CreateDate.ToLocalTime(),
                    user.LastLoginDate.ToLocalTime(), user.LastActivityDate.ToLocalTime(),
                    user.LastPasswordChangedDate.ToLocalTime(), user.LastLockoutDate.ToLocalTime());
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// ԴȡԱʸûϢ
        /// </summary>
        /// <param name="providerUserKey"></param>
        /// <param name="userIsOnline"></param>
        /// <returns></returns>
        public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
        {
            if (providerUserKey == null)
            {
                throw new ArgumentNullException("providerUserKey.");
            }
            if (!(providerUserKey is Int32))
            {
                throw new ArgumentException("Membership invalid provider userKey.");
            }

            User user = ServiceLocator.UserService.GetUserByUserID((int)providerUserKey, userIsOnline, DateTime.UtcNow);

            if (user != null)
            {
                return new MembershipUser(this.Name, user.Username, user.ID, user.Email,
                    user.PasswordQuestion, user.Comment, user.IsApproved, user.IsLockedOut, user.CreateDate.ToLocalTime(),
                    user.LastLoginDate.ToLocalTime(), user.LastActivityDate.ToLocalTime(),
                    user.LastPasswordChangedDate.ToLocalTime(), user.LastLockoutDate.ToLocalTime());
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// ȡָĵʼַû
        /// </summary>
        /// <param name="email"></param>
        /// <returns></returns>
        public override string GetUserNameByEmail(string email)
        {
            SecUtility.CheckParameter(ref email, false, false, false, 128, "email");

            IList users = ServiceLocator.UserService.GetUsersByEmail(this._applicationName, email);

            if (this._requiresUniqueEmail && users.Count > 0)
            {
                throw new ProviderException("Membership more than one_user with email.");

            }

            if (users.Count == 0)
            {
                return null;
            }

            return ((User)users[0]).Username;
        }

        /// <summary>
        /// ûΪһԶɵ
        /// </summary>
        /// <param name="username"></param>
        /// <param name="answer"></param>
        /// <returns></returns>
        public override string ResetPassword(string username, string passwordAnswer)
        {
            if (!this.EnablePasswordReset)
            {
                throw new NotSupportedException("Not configured to support password resets.");
            }

            SecUtility.CheckParameter(ref username, true, true, true, 255, "username");

            int status;
            string password;
            MembershipPasswordFormat passwordFormat;
            string salt;
            int failedPasswordAttemptCount;
            int failedPasswordAnswerAttemptCount;
            bool isApproved;
            DateTime lastLoginDate;
            DateTime lastActivityDate;

            GetPasswordWithFormat(this._applicationName, username, false, out status, out password, out passwordFormat,
                out salt, out failedPasswordAttemptCount, out failedPasswordAnswerAttemptCount, out isApproved,
                out lastLoginDate, out lastActivityDate);

            if (status == 0)
            {
                if (passwordAnswer != null)
                {
                    passwordAnswer = passwordAnswer.Trim();
                }

                string answer;

                if (!String.IsNullOrEmpty(passwordAnswer))
                {
                    answer = EncodePassword(passwordAnswer.ToLower(CultureInfo.InvariantCulture), passwordFormat, salt);
                }
                else
                {
                    answer = passwordAnswer;
                }

                SecUtility.CheckParameter(ref answer, this._requiresQuestionAndAnswer, this._requiresQuestionAndAnswer, false, 128, "passwordAnswer");

                string newPassword = GeneratePassword();

                ValidatePasswordEventArgs args = new ValidatePasswordEventArgs(username, newPassword, false);
                this.OnValidatingPassword(args);

                if (args.Cancel)
                {
                    if (args.FailureInformation != null)
                    {
                        throw args.FailureInformation;
                    }

                    throw new ProviderException("Membership custom password validation failure.");
                }

                status = ServiceLocator.UserService.ResetPassword(this._applicationName, username, newPassword,
                    this._maxInvalidPasswordAttempts, this._passwordAttemptWindow, salt, passwordFormat, answer,
                     DateTime.UtcNow);

                if (status != 0)
                {
                    string errText = this.GetExceptionText(status);

                    if (this.IsStatusDueToBadPassword(status))
                    {
                        throw new MembershipPasswordException(errText);
                    }

                    throw new ProviderException(errText);
                }

            }

            if (IsStatusDueToBadPassword(status))
            {
                throw new MembershipPasswordException(GetExceptionText(status));
            }

            throw new ProviderException(this.GetExceptionText(status));
        }

        /// <summary>
        /// û״̬Ա֤óԱʸû
        /// </summary>
        /// <param name="username"></param>
        /// <returns></returns>
        public override bool UnlockUser(string username)
        {
            SecUtility.CheckParameter(ref username, true, true, true, 255, "username");

            int status = ServiceLocator.UserService.UnlockUser(this._applicationName, username);

            if (status == 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// ³ԱʸݿûϢ
        /// </summary>
        /// <param name="user"></param>
        public override void UpdateUser(MembershipUser user)
        {
            if (user == null)
            {
                throw new ArgumentNullException("user.");
            }

            string email = user.Email;
            SecUtility.CheckParameter(ref email, this._requiresUniqueEmail, this._requiresUniqueEmail, false, 255, "email");
            user.Email = email;

            int status = ServiceLocator.UserService.UpdateUser(this._applicationName, user.UserName, user.Email,
                user.Comment, user.IsApproved, user.LastLoginDate.ToUniversalTime(), user.LastActivityDate.ToUniversalTime(),
                this._requiresUniqueEmail);

            if (status != 0)
            {
                throw new ProviderException(GetExceptionText(status));
            }
        }

        /// <summary>
        /// ֤ԱʸݿǷָû
        /// </summary>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        public override bool ValidateUser(string username, string password)
        {
            if ((SecUtility.ValidateParameter(ref username, true, true, true, 255) &&
                SecUtility.ValidateParameter(ref password, true, true, false, 128)) &&
                CheckPassword(username, password, true, true))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        #endregion 

        private bool IsStatusDueToBadPassword(int status)
        {
            if ((status >= 2) && (status <= 6))
            {
                return true;
            }
            return (status == 99);
        }

        private string GenerateSalt()
        {
            byte[] buffer = new byte[16];
            new RNGCryptoServiceProvider().GetBytes(buffer);
            return Convert.ToBase64String(buffer);
        }

        private DateTime RoundToSeconds(DateTime dt)
        {
            return new DateTime(dt.Year, dt.Month, dt.Day, dt.Hour, dt.Minute, dt.Second);
        }

        private string GetEncodedPasswordAnswer(string username, string passwordAnswer)
        {
            if (passwordAnswer != null)
            {
                passwordAnswer = passwordAnswer.Trim();
            }

            if (!String.IsNullOrEmpty(passwordAnswer))
            {
                int status;
                string password;
                MembershipPasswordFormat passwordFormat;
                string salt;
                int failedPasswordAttemptCount;
                int failedPasswordAnswerAttemptCount;
                bool isApproved;
                DateTime lastLoginDate;
                DateTime lastActivityDate;

                GetPasswordWithFormat(this._applicationName, username, false, out status, out password,
                    out passwordFormat, out salt, out failedPasswordAttemptCount, out failedPasswordAnswerAttemptCount,
                    out isApproved, out lastLoginDate, out lastActivityDate);

                if (status != 0)
                {
                    throw new ProviderException(GetExceptionText(status));
                }

                return EncodePassword(passwordAnswer.ToLower(CultureInfo.InvariantCulture), passwordFormat, salt);
            }

            return passwordAnswer;
        }

        private string GeneratePassword()
        {
            return Membership.GeneratePassword((this._minRequiredPasswordLength < 14) ? 14 : this._minRequiredPasswordLength,
                this._minRequiredNonAlphanumericCharacters);
        }


        private string EncodePassword(string pass, MembershipPasswordFormat passwordFormat, string salt)
        {
            if (passwordFormat == MembershipPasswordFormat.Clear)
            {
                return pass;
            }

            byte[] bIn = Encoding.Unicode.GetBytes(pass);
            byte[] bSalt = Convert.FromBase64String(salt);
            byte[] bAll = new byte[bSalt.Length + bIn.Length];
            byte[] bRet = null;

            Buffer.BlockCopy(bSalt, 0, bAll, 0, bSalt.Length);
            Buffer.BlockCopy(bIn, 0, bAll, bSalt.Length, bIn.Length);
            if (passwordFormat == MembershipPasswordFormat.Hashed)
            {
                HashAlgorithm s = HashAlgorithm.Create(Membership.HashAlgorithmType);

                if (s == null)
                {
                    throw new ProviderException("Could not create a hash algorithm");
                }
                bRet = s.ComputeHash(bAll);
            }
            else
            {
                bRet = base.EncryptPassword(bAll);
            }

            return Convert.ToBase64String(bRet);
        }

        private string UnEncodePassword(string pass, MembershipPasswordFormat passwordFormat)
        {
            switch (passwordFormat)
            {
                case MembershipPasswordFormat.Clear:
                    return pass;

                case MembershipPasswordFormat.Hashed:
                    throw new ProviderException("Provider can not decode hashed password");
            }

            byte[] buffer = this.DecryptPassword(Convert.FromBase64String(pass));

            if (buffer == null)
            {
                return null;
            }

            return Encoding.Unicode.GetString(buffer, 16, buffer.Length - 16);
        }

        private bool CheckPassword(string username, string password, bool updateLastLoginActivityDate, bool failIfNotApproved)
        {
            string salt;
            MembershipPasswordFormat passwordFormat;
            return CheckPassword(username, password, updateLastLoginActivityDate, failIfNotApproved,
                out salt, out passwordFormat);
        }

        private bool CheckPassword(string username, string password, bool updateLastLoginActivityDate, bool failIfNotApproved,
            out string salt, out MembershipPasswordFormat passwordFormat)
        {
            int status;
            string pass;
            int failedPasswordAttemptCount;
            int failedPasswordAnswerAttemptCount;
            bool isApproved;
            DateTime lastLoginDate;
            DateTime lastActivityDate;


            GetPasswordWithFormat(this._applicationName, username, updateLastLoginActivityDate, out status,
                out pass, out passwordFormat, out salt, out failedPasswordAttemptCount,
                out failedPasswordAnswerAttemptCount, out isApproved, out lastLoginDate, out  lastActivityDate);

            if (status != 0)
            {
                return false;
            }
            if (!isApproved && failIfNotApproved)
            {
                return false;
            }

            password = EncodePassword(password, passwordFormat, salt);

            bool isPasswordCorrect = password.Equals(pass);

            if ((isPasswordCorrect && (failedPasswordAttemptCount == 0)) && (failedPasswordAnswerAttemptCount == 0))
            {
                return true;
            }

            ServiceLocator.UserService.UpdateUserInfo(this._applicationName, username, isPasswordCorrect,
                this._passwordAttemptWindow, this._maxInvalidPasswordAttempts,
                updateLastLoginActivityDate, DateTime.UtcNow);

            return isPasswordCorrect;

        }

        private string GetExceptionText(int status)
        {
            string errText;
            switch (status)
            {
                case 0:
                    return String.Empty;

                case 1:
                    errText = "Membership user not found.";
                    break;

                case 2:
                    errText = "Membership wrong password.";
                    break;

                case 3:
                    errText = "Membership wrong answer.";
                    break;

                case 4:
                    errText = "Membership invalid password.";
                    break;

                case 5:
                    errText = "Membership invalid question.";
                    break;

                case 6:
                    errText = "Membership invalid answer.";
                    break;

                case 7:
                    errText = "Membership invalid email.";
                    break;

                case 99:
                    errText = "Membership account lock out.";
                    break;

                default:
                    errText = "Provider error.";
                    break;
            }
            return errText;
        }

        private int CreateUser(string applicationName, string username, string password, string passwordSalt,
            string email, string passwordQuestion, string passwordAnswer, bool isApproved, bool uniqueEmail,
            MembershipPasswordFormat passwordFormat, DateTime currentTimeUtc, out int userID)
        {
            userID = -1;


            if (ServiceLocator.UserService.GetUserByName(applicationName, username) != null)
            {
                return 6; //MembershipCreateStatus.DuplicateUserName
            }

            if (uniqueEmail && ServiceLocator.UserService.GetUsersByEmail(applicationName, email).Count > 0)
            {
                return 7; //MembershipCreateStatus.DuplicateEmail
            }

            User user = new User();
            user.Username = username;
            user.Email = email;
            user.Password = password;
            user.PasswordFormat = passwordFormat;
            user.PasswordSalt = passwordSalt;
            user.PasswordQuestion = passwordQuestion;
            user.PasswordAnswer = passwordAnswer;
            user.IsAnonymous = false;
            user.IsApproved = isApproved;
            user.IsLockedOut = false;
            user.CreateDate = currentTimeUtc;
            user.LastActivityDate = currentTimeUtc;
            user.LastLoginDate = currentTimeUtc;
            user.LastPasswordChangedDate = currentTimeUtc;
            user.LastLockoutDate = new DateTime(1754, 1, 1);
            user.FailedPasswordAttemptCount = 0;
            user.FailedPasswordAttemptWindowStart = new DateTime(1754, 1, 1);
            user.FailedPasswordAnswerAttemptCount = 0;
            user.FailedPasswordAnswerAttemptWindowStart = new DateTime(1754, 1, 1);
            user.Application = ServiceLocator.ApplicationService.GetApplication(applicationName);

            ServiceLocator.UserService.CreateUser(user);

            if (user.ID > 0)
            {
                userID = user.ID;
                return 0;//MembershipCreateStatus.Success
            }
            else
            {
                return 11;//MembershipCreateStatus.ProviderError
            }
        }


        private void GetPasswordWithFormat(string applicationName, string username, bool updateLastLoginActivityDate, out int status,
             out string password, out MembershipPasswordFormat passwordFormat, out string passwordSalt,
             out int failedPasswordAttemptCount, out int failedPasswordAnswerAttemptCount, out bool isApproved,
             out DateTime lastLoginDate, out DateTime lastActivityDate)
        {
            User user = ServiceLocator.UserService.GetUserByName(applicationName, username);

            if (user != null)
            {
                if (updateLastLoginActivityDate && user.IsApproved)
                {
                    user.LastActivityDate = DateTime.UtcNow;
                    user.LastLoginDate = DateTime.UtcNow;

                    ServiceLocator.UserService.UpdateUser(user);
                }

                password = user.Password;
                passwordFormat = user.PasswordFormat;
                passwordSalt = user.PasswordSalt;
                failedPasswordAttemptCount = user.FailedPasswordAttemptCount;
                failedPasswordAnswerAttemptCount = user.FailedPasswordAnswerAttemptCount;
                isApproved = user.IsApproved;
                lastLoginDate = user.LastLoginDate;
                lastActivityDate = user.LastActivityDate;

                status = 0;
            }
            else
            {
                password = null;
                passwordFormat = MembershipPasswordFormat.Clear;
                passwordSalt = null;
                failedPasswordAttemptCount = 0;
                failedPasswordAnswerAttemptCount = 0;
                isApproved = false;
                lastLoginDate = DateTime.UtcNow;
                lastActivityDate = DateTime.UtcNow;

                status = 1;
            }
        }
    }
}
