using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Security;
using System.Collections.Specialized;
using System.Configuration.Provider;
using Guushuuse.SalaryPrj.Security.Service;

namespace Guushuuse.SalaryPrj.Security.Providers
{
    /// <summary>
    /// ݿ ASP.NET ӦóĳԱʸϢ洢
    /// </summary>
    public class HibernateRoleProvider : RoleProvider
    {
        private string _applicationName;

        #region 

        /// <summary>
        /// ȡҪ洢ͼɫϢӦó
        /// </summary>
        public override string ApplicationName
        {
            get
            {
                return _applicationName;
            }
            set
            {
                if (String.IsNullOrEmpty(value))
                {
                    throw new ArgumentNullException("Provider application name not null.");
                }
                if (value.Length > 255)
                {
                    throw new ProviderException("Provider application name too long.");
                }
                _applicationName = value;

            }
        }

        #endregion 

        #region 

        /// <summary>
        ///  ASP.NET Ӧóļֵָʼɫṩ
        /// </summary>
        /// <param name="name"></param>
        /// <param name="config"></param>
        public override void Initialize(string name, NameValueCollection config)
        {
            if (config == null)
            {
                throw new ArgumentNullException("config");
            }

            if (String.IsNullOrEmpty(name))
            {
                name = "HibernateRoleProvider";
            }

            if (String.IsNullOrEmpty(config["description"]))
            {
                config.Remove("description");
                config.Add("description", "GFC.Security Role Provider");
            }

            base.Initialize(name, config);


            this._applicationName = config["applicationName"];
            if (String.IsNullOrEmpty(this._applicationName))
            {
                this._applicationName = SecUtility.GetDefaultAppName();
            }

            if (this._applicationName.Length > 255)
            {
                throw new ProviderException("Provider application name is too long, max length is 255.");
            }

            if (!ServiceLocator.ApplicationService.ApplicationExists(this._applicationName))
            {
                ServiceLocator.ApplicationService.CreateApplication(this._applicationName);
            }

            config.Remove("applicationName");


            if (config.Count > 0)
            {
                string attribUnrecognized = config.GetKey(0);
                if (!String.IsNullOrEmpty(attribUnrecognized))
                {
                    throw new ProviderException("Provider unrecognized attribute: " + attribUnrecognized);
                }
            }
        }

        /// <summary>
        /// ָûӵÿָĽɫ 
        /// </summary>
        /// <param name="usernames"></param>
        /// <param name="roleNames"></param>
        public override void AddUsersToRoles(string[] usernames, string[] roleNames)
        {
            SecUtility.CheckArrayParameter(ref usernames, true, true, true, 255, "usernames");
            SecUtility.CheckArrayParameter(ref roleNames, true, true, true, 255, "roleNames");

            int status = ServiceLocator.RoleService.AddUsersToRoles(this._applicationName, usernames, roleNames);

            if (status != 0)
            {
                string errText = this.GetExceptionText(status);

                throw new ProviderException(errText);
            }

        }

        /// <summary>
        /// µĽɫӵɫݿ⡣
        /// </summary>
        /// <param name="roleName"></param>
        public override void CreateRole(string roleName)
        {
            SecUtility.CheckParameter(ref roleName, true, true, true, 255, "roleName");

            int status = ServiceLocator.RoleService.CreateRole(this._applicationName, roleName);

            if (status != 0)
            {
                string errText = this.GetExceptionText(status);

                throw new ProviderException(errText);
            }
        }

        /// <summary>
        /// ӽɫݿƳһɫ
        /// </summary>
        /// <param name="roleName"></param>
        /// <param name="throwOnPopulatedRole"></param>
        /// <returns></returns>
        public override bool DeleteRole(string roleName, bool throwOnPopulatedRole)
        {
            SecUtility.CheckParameter(ref roleName, true, true, true, 255, "roleName");

            int status = ServiceLocator.RoleService.DeleteRole(this._applicationName, roleName, throwOnPopulatedRole);

            if (status != 0)
            {


                return false;
            }

            return true;
        }

        /// <summary>
        /// ȡĳɫָûƥû
        /// </summary>
        /// <param name="roleName"></param>
        /// <param name="usernameToMatch"></param>
        /// <returns></returns>
        public override string[] FindUsersInRole(string roleName, string usernameToMatch)
        {
            SecUtility.CheckParameter(ref roleName, true, true, true, 255, "roleName");
            SecUtility.CheckParameter(ref usernameToMatch, true, true, false, 255, "usernameToMatch");

            return ServiceLocator.RoleService.FindUsersInRole(this._applicationName, roleName, usernameToMatch);
        }

        /// <summary>
        /// ȡӦóнɫб
        /// </summary>
        /// <returns></returns>
        public override string[] GetAllRoles()
        {
            return ServiceLocator.RoleService.GetAllRoles(this._applicationName);
        }

        /// <summary>
        /// ȡһûɫб
        /// </summary>
        /// <param name="username"></param>
        /// <returns></returns>
        public override string[] GetRolesForUser(string username)
        {
            SecUtility.CheckParameter(ref username, true, false, true, 255, "username");

            return ServiceLocator.RoleService.GetRolesForUser(this._applicationName, username);
        }

        /// <summary>
        /// ȡָɫûб
        /// </summary>
        /// <param name="roleName"></param>
        /// <returns></returns>
        public override string[] GetUsersInRole(string roleName)
        {
            SecUtility.CheckParameter(ref roleName, true, true, true, 255, "roleName");

            return ServiceLocator.RoleService.GetUsersInRole(this._applicationName, roleName);
        }

        /// <summary>
        /// ȡһָʾָûǷָɫֵ
        /// </summary>
        /// <param name="username"></param>
        /// <param name="roleName"></param>
        /// <returns></returns>
        public override bool IsUserInRole(string username, string roleName)
        {
            SecUtility.CheckParameter(ref roleName, true, true, true, 255, "roleName");
            SecUtility.CheckParameter(ref username, true, false, true, 255, "username");

            return ServiceLocator.RoleService.IsUserInRole(this._applicationName, username, roleName);
        }

        /// <summary>
        /// Ƴָɫеָû
        /// </summary>
        /// <param name="usernames"></param>
        /// <param name="roleNames"></param>
        public override void RemoveUsersFromRoles(string[] usernames, string[] roleNames)
        {
            SecUtility.CheckArrayParameter(ref usernames, true, true, true, 255, "usernames");
            SecUtility.CheckArrayParameter(ref roleNames, true, true, true, 255, "roleNames");

            int status = ServiceLocator.RoleService.RemoveUsersFromRoles(this._applicationName, usernames, roleNames);

            if (status != 0)
            {
                string errText = this.GetExceptionText(status);

                throw new ProviderException(errText);
            }

        }

        /// <summary>
        /// ȡһֵֵָʾָĽɫǷѴڽɫݿ
        /// </summary>
        /// <param name="roleName"></param>
        /// <returns></returns>
        public override bool RoleExists(string roleName)
        {
            SecUtility.CheckParameter(ref roleName, true, true, true, 255, "roleName");

            return ServiceLocator.RoleService.RoleExists(this._applicationName, roleName);
        }

        #endregion 


        private string GetExceptionText(int status)
        {
            string errText;
            switch (status)
            {
                case 0:
                    return String.Empty;

                case 1:
                    errText = "User not found.";
                    break;

                case 2:
                    errText = "Role not found.";
                    break;

                case 3:
                    errText = "This user already in role.";
                    break;

                case 4:
                    errText = "Role is not empty.";
                    break;

                case 5:
                    errText = "Role already exists.";
                    break;

                default:
                    errText = "Provider error.";
                    break;
            }
            return errText;
        }
    }
}
